searchNodes=[{"ref":"JDDF.html","title":"JDDF","type":"module","doc":"An Elixir implementation of JSON Data Definition Format, a schema language for JSON. You can use this module to parse JDDF schemas, and then validate JSON data against those schemas. JDDF doesn&#39;t parse JSON data. Instead, you should first parse JSON data, and then call JDDF with the parsed result. JDDF is compatible with the sort of data produced by Jason or Poison. You can also just pass in ordinary Elixir data structures, as the examples here will demonstrate. Example For example, let&#39;s parse the following JDDF schema: { &quot;properties&quot;: { &quot;name&quot;: { &quot;type&quot;: &quot;string&quot; }, &quot;age&quot;: { &quot;type&quot;: &quot;string&quot; }, &quot;phones&quot;: { &quot;elements&quot;: { &quot;type&quot;: &quot;string&quot; } } } } And validate it against two inputs. The first passes all the validation rules: { &quot;name&quot;: &quot;John Doe&quot;, &quot;age&quot;: 42, &quot;phones&quot;: [ &quot;+44 1234567&quot;, &quot;+44 2345678&quot;, ] } But the second does not: { &quot;age&quot;: &quot;42&quot;, &quot;phones&quot;: [ &quot;+44 1234567&quot;, 442345678, ] } With this second input, there are three problems: name is missing age has the wrong type phones[1] has the wrong type Let&#39;s see an example of how we would parse the schema, and then validate the two inputs against it. We&#39;ll expect to get back no errors for the first input, and we expect three validation errors for the second input: To keep things simple, we&#39;ll use Elixir data structures directly in these examples. In real life, you&#39;d probably parse this data using Jason.decode! or Poison.decode!. iex&gt; schema = JDDF.Schema.from_json!(%{ ...&gt; &quot;properties&quot; =&gt; %{ ...&gt; &quot;name&quot; =&gt; %{&quot;type&quot; =&gt; &quot;string&quot;}, ...&gt; &quot;age&quot; =&gt; %{&quot;type&quot; =&gt; &quot;uint32&quot;}, ...&gt; &quot;phones&quot; =&gt; %{ ...&gt; &quot;elements&quot; =&gt; %{&quot;type&quot; =&gt; &quot;string&quot;}, ...&gt; }, ...&gt; }, ...&gt; }) iex&gt; validator = %JDDF.Validator{} %JDDF.Validator{max_depth: 0, max_errors: 0} iex&gt; JDDF.Validator.validate!(validator, schema, %{ ...&gt; &quot;name&quot; =&gt; &quot;John Doe&quot;, ...&gt; &quot;age&quot; =&gt; 42, ...&gt; &quot;phones&quot; =&gt; [ ...&gt; &quot;+44 1234567&quot;, ...&gt; &quot;+44 2345678&quot;, ...&gt; ], ...&gt; }) [] iex&gt; errors = Enum.sort(JDDF.Validator.validate!(validator, schema, %{ ...&gt; &quot;age&quot; =&gt; &quot;42&quot;, ...&gt; &quot;phones&quot; =&gt; [ ...&gt; &quot;+44 1234567&quot;, ...&gt; 442345678, ...&gt; ], ...&gt; })) iex&gt; length(errors) 3 iex&gt; [e1, e2, e3] = errors iex&gt; e1 %JDDF.Validator.ValidationError{instance_path: [], schema_path: [&quot;properties&quot;, &quot;name&quot;]} iex&gt; e2 %JDDF.Validator.ValidationError{instance_path: [&quot;age&quot;], schema_path: [&quot;properties&quot;, &quot;age&quot;, &quot;type&quot;]} iex&gt; e3 %JDDF.Validator.ValidationError{instance_path: [&quot;phones&quot;, &quot;1&quot;], schema_path: [&quot;properties&quot;, &quot;phones&quot;, &quot;elements&quot;, &quot;type&quot;]}"},{"ref":"JDDF.Schema.html","title":"JDDF.Schema","type":"module","doc":"Represents a JSON Data Definition Format schema. You can construct instances of this type yourself, or you can parse it from JSON-like data using from_json/1 or from_json!/1. Not all invariants that JDDF schemas must adhere to are checked by from_json/1 or from_json!/1. To fully verify the correctness of a schema, use verify/1 or verify!/1."},{"ref":"JDDF.Schema.html#from_json/1","title":"JDDF.Schema.from_json/1","type":"function","doc":"Construct a JDDF schema from parsed JSON data. json should be an Elixir representation of JSON data. You should first parse the JSON before passing it into from_json/1. You can construct this data manually, or parse it using: Jason.decode!/2, or Poison.decode!/2 Most JSON implementations for Elixir are compatible with this function. iex&gt; JDDF.Schema.from_json(%{&quot;type&quot; =&gt; &quot;uint32&quot;}) {:ok, %JDDF.Schema{definitions: nil, form: {:type, :uint32}}} iex&gt; JDDF.Schema.from_json(%{&quot;type&quot; =&gt; &quot;nonsense&quot;}) {:error, %JDDF.Schema.InvalidSchemaError{message: &quot;invalid type&quot;}}"},{"ref":"JDDF.Schema.html#from_json!/1","title":"JDDF.Schema.from_json!/1","type":"function","doc":"Construct a JDDF schema from parsed JSON data. Similar to from_json/1, except it will unwrap the result and will raise in case of errors. iex&gt; JDDF.Schema.from_json!(%{&quot;type&quot; =&gt; &quot;uint32&quot;}) %JDDF.Schema{definitions: nil, form: {:type, :uint32}} iex&gt; JDDF.Schema.from_json!(%{&quot;type&quot; =&gt; &quot;nonsense&quot;}) ** (JDDF.Schema.InvalidSchemaError) invalid type"},{"ref":"JDDF.Schema.html#verify/1","title":"JDDF.Schema.verify/1","type":"function","doc":"Ensure that the invariants of a JDDF schema are adhered to. iex&gt; JDDF.Schema.verify(JDDF.Schema.from_json!(%{})) {:ok, nil} iex&gt; schema = JDDF.Schema.from_json!(%{ ...&gt; &quot;properties&quot; =&gt; %{ &quot;a&quot; =&gt; %{}}, ...&gt; &quot;optionalProperties&quot; =&gt; %{ &quot;a&quot; =&gt; %{}}, ...&gt; }) iex&gt; JDDF.Schema.verify(schema) {:error, %JDDF.Schema.InvalidSchemaError{message: &quot;properties and optionalProperties share key&quot;}}"},{"ref":"JDDF.Schema.html#verify!/1","title":"JDDF.Schema.verify!/1","type":"function","doc":"Ensure that the invariants of a JDDF schema are adhered to. Similar to verify/1, except it will unwrap the result and will raise in case of errors. iex&gt; JDDF.Schema.verify!(JDDF.Schema.from_json!(%{})) nil iex&gt; schema = JDDF.Schema.from_json!(%{ ...&gt; &quot;properties&quot; =&gt; %{ &quot;a&quot; =&gt; %{}}, ...&gt; &quot;optionalProperties&quot; =&gt; %{ &quot;a&quot; =&gt; %{}}, ...&gt; }) iex&gt; JDDF.Schema.verify!(schema) ** (JDDF.Schema.InvalidSchemaError) properties and optionalProperties share key"},{"ref":"JDDF.Schema.html#t:form/0","title":"JDDF.Schema.form/0","type":"type","doc":""},{"ref":"JDDF.Schema.html#t:mapping/0","title":"JDDF.Schema.mapping/0","type":"type","doc":""},{"ref":"JDDF.Schema.html#t:t/0","title":"JDDF.Schema.t/0","type":"type","doc":""},{"ref":"JDDF.Schema.html#t:type/0","title":"JDDF.Schema.type/0","type":"type","doc":""},{"ref":"JDDF.Schema.InvalidSchemaError.html","title":"JDDF.Schema.InvalidSchemaError","type":"exception","doc":"Indicates that a schema does not conform to the JDDF syntax rules."},{"ref":"JDDF.Schema.InvalidSchemaError.html#t:t/0","title":"JDDF.Schema.InvalidSchemaError.t/0","type":"type","doc":""},{"ref":"JDDF.Validator.html","title":"JDDF.Validator","type":"module","doc":"Validates JSON data against instances of JDDF.Schema. Validators have two properties to control how validation behaves: max_depth controls how &quot;deeply&quot; references in JDDF schemas (using the ref JDDF keyword) should be followed. If max_depth is exceeded, then validate/3 or validate!/3 will throw JDDF.Validator.MaxDepthExceededError. For example, setting max_depth to 5 means that a 5-deep chain of refs would trigger JDDF.Validator.MaxDepthExceededError. By default, max_depth is 0. A max_depth of 0 disables any maximum depth; JDDF.Validator.MaxDepthExceededError will never be thrown. In this case, validating against a circularly-defined schema may exhaust all memory. max_errors controls the maximum number of errors to return. If max_errors is set to N, then validate/3 and validate!/3 will return NJDDF.Validator.ValidationErrors or fewer. You can use max_errors to optimize JDDF validation. For example, if you only care about whether there exist any validation errors for an input, you can set max_errors to 1. By default, max_errors is 0. A max_errors of 0 disables any maximum number of errors; validate/3 and validate!/3 will return all errors. The examples below demonstrate max_depth and max_errors. See also the docs for JDDF for more introductory examples. Examples Here&#39;s an example of max_depth causing an error, rather than letting a circularly-defined schema cause an infinite loop: iex&gt; schema = JDDF.Schema.from_json!(%{ ...&gt; &quot;definitions&quot; =&gt; %{ ...&gt; &quot;loop&quot; =&gt; %{ &quot;ref&quot; =&gt; &quot;loop&quot; }, ...&gt; }, ...&gt; &quot;ref&quot; =&gt; &quot;loop&quot;, ...&gt; }) iex&gt; validator = %JDDF.Validator{max_depth: 32} %JDDF.Validator{max_depth: 32, max_errors: 0} iex&gt; JDDF.Validator.validate!(validator, schema, nil) ** (JDDF.Validator.MaxDepthExceededError) maximum depth exceeded during validation Here&#39;s an example of max_errors limiting the return value to just 1 value: iex&gt; schema = JDDF.Schema.from_json!(%{ ...&gt; &quot;elements&quot; =&gt; %{ &quot;type&quot; =&gt; &quot;string&quot; } ...&gt; }) iex&gt; validator = %JDDF.Validator{max_errors: 1} %JDDF.Validator{max_depth: 0, max_errors: 1} iex&gt; length(JDDF.Validator.validate!(validator, schema, [nil, nil, nil, nil, nil])) 1 Instead of returning five errors (one for each element of [nil, nil, nil, nil, nil]), just one error comes back. Internally, validate/3 and validate!/3 optimizes for max_errors, and stops validation as soon as max_errors is reached."},{"ref":"JDDF.Validator.html#validate/3","title":"JDDF.Validator.validate/3","type":"function","doc":"Validate a JDDF.Schema against a JSON input (an &quot;instance&quot;). instance should be an Elixir representation of JSON data. You should first parse the JSON before passing it into validate/3. You can construct this data manually, or parse it using: Jason.decode!/2, or Poison.decode!/2 Most JSON implementations for Elixir are compatible with this function. See JDDF.Validator docs for how to control certain aspects of this function. This function implements the formal specification of JSON Data Definition Format validation. iex&gt; JDDF.Validator.validate( ...&gt; %JDDF.Validator{}, ...&gt; JDDF.Schema.from_json!(%{&quot;type&quot; =&gt; &quot;boolean&quot;}), ...&gt; nil ...&gt; ) {:ok, [%JDDF.Validator.ValidationError{instance_path: [], schema_path: [&quot;type&quot;]}]} iex&gt; schema = JDDF.Schema.from_json!(%{ ...&gt; &quot;definitions&quot; =&gt; %{ ...&gt; &quot;loop&quot; =&gt; %{ &quot;ref&quot; =&gt; &quot;loop&quot; }, ...&gt; }, ...&gt; &quot;ref&quot; =&gt; &quot;loop&quot;, ...&gt; }) iex&gt; validator = %JDDF.Validator{max_depth: 32} %JDDF.Validator{max_depth: 32, max_errors: 0} iex&gt; JDDF.Validator.validate(validator, schema, nil) {:error, %JDDF.Validator.MaxDepthExceededError{message: &quot;maximum depth exceeded during validation&quot;}}"},{"ref":"JDDF.Validator.html#validate!/3","title":"JDDF.Validator.validate!/3","type":"function","doc":"Validate a JDDF.Schema against a JSON input (an &quot;instance&quot;). Similar to validate/3, except it will unwrap the result and will raise in case of errors. iex&gt; JDDF.Validator.validate!( ...&gt; %JDDF.Validator{}, ...&gt; JDDF.Schema.from_json!(%{&quot;type&quot; =&gt; &quot;boolean&quot;}), ...&gt; nil ...&gt; ) [%JDDF.Validator.ValidationError{instance_path: [], schema_path: [&quot;type&quot;]}] iex&gt; schema = JDDF.Schema.from_json!(%{ ...&gt; &quot;definitions&quot; =&gt; %{ ...&gt; &quot;loop&quot; =&gt; %{ &quot;ref&quot; =&gt; &quot;loop&quot; }, ...&gt; }, ...&gt; &quot;ref&quot; =&gt; &quot;loop&quot;, ...&gt; }) iex&gt; validator = %JDDF.Validator{max_depth: 32} %JDDF.Validator{max_depth: 32, max_errors: 0} iex&gt; JDDF.Validator.validate!(validator, schema, nil) ** (JDDF.Validator.MaxDepthExceededError) maximum depth exceeded during validation"},{"ref":"JDDF.Validator.html#t:t/0","title":"JDDF.Validator.t/0","type":"type","doc":""},{"ref":"JDDF.Validator.MaxDepthExceededError.html","title":"JDDF.Validator.MaxDepthExceededError","type":"exception","doc":"Indicates that the validator&#39;s max depth was exceeded during validation. This error is raised when a JDDF.Validator&#39;s max_depth is exceeded while doing validation, through either JDDF.Validator.validate/3 or JDDF.Validator.validate!/3. See docs for JDDF.Validator for further explanation of max_depth, and why this error may be considered desirable."},{"ref":"JDDF.Validator.MaxDepthExceededError.html#t:t/0","title":"JDDF.Validator.MaxDepthExceededError.t/0","type":"type","doc":""},{"ref":"JDDF.Validator.ValidationError.html","title":"JDDF.Validator.ValidationError","type":"module","doc":"Represents a single JDDF validation error. instance_path is the path to the part of the input that was rejected. schema_path is the path the part of the schema that rejected the input. The precise values that go into instance_path and schema_path is standardized. The JDDF specification formalizes what exactly needs to go into these fields. JDDF.Validator confirms to the JDDF specification."},{"ref":"JDDF.Validator.ValidationError.html#t:t/0","title":"JDDF.Validator.ValidationError.t/0","type":"type","doc":""},{"ref":"readme.html","title":"jddf-elixir ![][hex-badge]][hex-url] [![][ci-badge]","type":"extras","doc":"jddf-elixir ![][hex-badge]][hex-url] [![][ci-badge] Documentation on Hexdocs: https://hexdocs.pm/jddf This package is an Elixir implementation of JSON Data Definition Format. You can use this package to: Validate input data against a schema, Get a list of validation errors from that input data, or Build your own tooling on top of JSON Data Definition Format"},{"ref":"readme.html#installation","title":"jddf-elixir ![][hex-badge]][hex-url] [![][ci-badge] - Installation","type":"extras","doc":"Add JDDF to your mix.exs dependencies: def deps do [ {:jddf, &quot;~&gt; 0.1.0&quot;} ] end Then update your dependencies: mix deps.get"},{"ref":"readme.html#usage","title":"jddf-elixir ![][hex-badge]][hex-url] [![][ci-badge] - Usage","type":"extras","doc":"See the docs for more detailed usage, but at a high level, here&#39;s how you parse schemas and validate input data against them: schema = JDDF.Schema.from_json!(%{ &quot;properties&quot; =&gt; %{ &quot;name&quot; =&gt; %{&quot;type&quot; =&gt; &quot;string&quot;}, &quot;age&quot; =&gt; %{&quot;type&quot; =&gt; &quot;uint32&quot;}, &quot;phones&quot; =&gt; %{ &quot;elements&quot; =&gt; %{&quot;type&quot; =&gt; &quot;string&quot;}, }, }, }) validator = %JDDF.Validator{} # To keep this example simple, we&#39;ll construct this data by hand. But you # could also parse this data from JSON. # # This input data is perfect. It satisfies all the schema requirements. errors_ok = JDDF.Validator.validate!(validator, schema, %{ &quot;name&quot; =&gt; &quot;John Doe&quot;, &quot;age&quot; =&gt; 42, &quot;phones&quot; =&gt; [ &quot;+44 1234567&quot;, &quot;+44 2345678&quot;, ], }) IO.inspect(errors_ok) # Outputs: [] # This input data has problems. &quot;name&quot; is missing, &quot;age&quot; has the wrong type, # and &quot;phones[1]&quot; has the wrong type. # # To make the output predictable for this example, we&#39;ll sort the errors. You # don&#39;t have to do this in your applications. errors_bad = Enum.sort(JDDF.Validator.validate!(validator, schema, %{ &quot;age&quot; =&gt; &quot;42&quot;, &quot;phones&quot; =&gt; [ &quot;+44 1234567&quot;, 442345678, ], })) IO.inspect(length(errors_bad)) # 3 [e1, e2, e3] = errors_bad IO.inspect(e1) # Outputs: %JDDF.Validator.ValidationError{instance_path: [], schema_path: [&quot;properties&quot;, &quot;name&quot;]} # # This error indicates that the root is missing &quot;name&quot; IO.inspect(e2) # Outputs: %JDDF.Validator.ValidationError{instance_path: [&quot;age&quot;], schema_path: [&quot;properties&quot;, &quot;age&quot;, &quot;type&quot;]} # # This error indicates that &quot;age&quot; has the wrong type IO.inspect(e3) # Outputs: %JDDF.Validator.ValidationError{instance_path: [&quot;phones&quot;, &quot;1&quot;], schema_path: [&quot;properties&quot;, &quot;phones&quot;, &quot;elements&quot;, &quot;type&quot;]} # # This error indicates that &quot;phones[1]&quot; has the wrong type"}]